@@ -31,219 +31,236 @@ test = function(x, y, type = "=", tol = 1e-6){
 
   type = switch(type, "error" = "err", "warning" = "warn", type)
 
-  plural_core = function(PLURAL, type, s, verb = FALSE, past = FALSE){
-
-    if(!missing(type)){
-      args = strsplit(type, "\\.")[[1]]
-      s = "s" %in% args
-      past = "past" %in% args
-      verb = setdiff(args, c("s", "past"))
-      if(length(verb) == 0){
-        verb = FALSE
-      } else {
-        verb = verb[1]
-      }
-    }
-
-    if(isFALSE(verb)){
-      res = ifelse(PLURAL, "s", "")
-    } else {
-
-      if(verb %in% c("be", "are")){
-        verb = "is"
-      } else if(verb == "have"){
-        verb = "has"
-      } else if(verb == "does"){
-        verb = "do"
-      } else if(verb %in% c("do not", "does not", "don't", "doesn't")){
-        verb = "do not"
-      } else if(verb %in% c("is not", "are not", "isn't", "aren't")){
-        verb = "is not"
-      } else if(verb %in% c("was", "were")){
-        verb = "is"
-        past = TRUE
-      }
-
-      if(past){
-        if(verb %in% c("is", "is not", "has", "do", "do not")){
-          verb_format = switch(verb, is = ifelse(!PLURAL, "was", "were"), "is not" = ifelse(!PLURAL, "wasn't", "weren't"), has = "had", do = "did", "do not" = "didn't")
-        } else {
-          verb_format = paste0(verb, "ed")
-        }
-      } else {
-        if(verb %in% c("is", "is not", "has", "do", "do not")){
-          verb_format = switch(verb, is = ifelse(!PLURAL, "is", "are"), "is not" = ifelse(!PLURAL, "isn't", "aren't"), has = ifelse(!PLURAL, "has", "have"), do = ifelse(!PLURAL, "does", "do"), "do not" = ifelse(!PLURAL, "doesn't", "don't"))
-        } else {
-          verb_format = ifelse(PLURAL, verb, paste0(verb, "s"))
-        }
-      }
-
-      if(!missing(s) && isTRUE(s)){
-        res = paste0(ifelse(PLURAL, "s ", " "), verb_format)
-      } else {
-        res = verb_format
-      }
-
-    }
-
-    res
-  }
-
-
-  plural = function(x, type, s, verb = FALSE, past = FALSE){
-    # adds s if x > 1, can alternatively add a verb
-    PLURAL = x[1] > 1
-    plural_core(PLURAL, type, s, verb, past)
-  }
-
-  n_th = function(n){
-    if(length(n) == 0) return("")
-    dict = c("first", "second", "third", "fourth", "fifth", "sixth", "seventh", "eighth", "nineth", "tenth", "eleventh", "twelfth", "thirteenth")
-    res = as.character(n)
-    qui = n <= 13
-    res[qui] = dict[n[qui]]
-    if(any(!is.na(qui) & !qui)){
-      other = n[!qui]
-      rest = other %% 10
-      rest[rest == 0 | rest >= 4] = 4
-      postfix = c("st", "nd", "rd", "th")
-      other = paste0(other, postfix[rest])
-      res[!qui] = other
-    }
-
-    res
-  }
-
   if(type == "=" && is.numeric(x)){
     type = "~"
     if(missing(tol)) tol = 1e-12
   }
   
+  actual   = "\n  actual: "
+  expected = "\nexpected: "
+  
+  fmt_num = function(x){
+    format(x, big.mark = ",")
+  }
+  
+  fmt_xy = function(x, y){
+    
+    n_x = length(x)
+    n_y = length(y)
+    n = max(n_x, n_y)
+    do_clean = FALSE
+    if(length(x) != length(y)){
+      do_clean = TRUE
+      x = x[1:n]
+      y = y[1:n]
+    }
+    
+    if(n > 15){
+      x = x[1:15]
+      y = y[1:15]
+    }
+    
+    n_x_fmt = min(n_x, 15)
+    n_y_fmt = min(n_y, 15)
+    n_fmt = max(n_x_fmt, n_y_fmt)
+    
+    width = getOption("width", 100) * 0.95 - nchar(actual)
+    
+    all_xy = rbind(x, y)
+    all_xy_fmt = apply(all_xy, 2, format, big.mark = ",", simplify = TRUE)
+    
+    if(do_clean){
+      n_diff = abs(n_x_fmt - n_y_fmt)
+      if(n_diff > 0){
+        if(n_x < n_y){
+          is_clean = 1:n_diff + n_x + 1 
+          all_xy_fmt[1, -is_clean] = gsub(".", " ", all_xy_fmt[1, -is_clean])
+        } else {
+          is_clean = 1:n_diff + n_y + 1 
+          all_xy_fmt[2, -is_clean] = gsub(".", " ", all_xy_fmt[2, -is_clean])
+        }
+      }
+    }
+    
+    all_widths = cumsum(nchar(all_xy_fmt[1, ])) + cumsum(rep(2, n_fmt))
+    
+    if(tail(all_widths, 1) < width){
+      lines_fmt = apply(all_xy_fmt, 1, paste, collapse = "  ")
+      res = paste0(actual, lines_fmt[1], expected, lines_fmt[2])
+      return(res)
+    }
+    
+    # we need to make it fit
+    extra = "  [5 others]"
+    width = width - nchar(extra) - ceiling(log10(n) - 1)
+    
+    all_ok = which(all_widths <= width)
+    
+    if(length(all_ok) == 0){
+      res = paste0(actual,   "x[1] = ", x[1], 
+                   expected, "y[1] = ", y[1])
+      return(res)
+    }
+    
+    k = tail(all_ok, 1)
+    extra_x = sma("  [{n ? n_x - k} others]")
+    extra_y = sma("  [{n ? n_y - k} others]")
+    lines_fmt = apply(all_xy_fmt[, 1:k, drop = FALSE], 1, paste, collapse = "  ")
+    
+    res = paste0(actual,   lines_fmt[1], extra_x, 
+                 expected, lines_fmt[2], extra_y)
+    return(res)
+  }
+  
   if(type  == "warn"){
     # we expect an warning
     m = tryCatch(x, warning = function(w) structure(conditionMessage(w), class = "try-warning"))
     if(!"try-warning" %in% class(m)){
       stop("Expected a warning that did not occur.")
+      
     } else if(IS_Y && !grepl(tolower(y), tolower(m), fixed = TRUE)){
-      stop("This is a warning, but the messages don't match:\nEXPECTED: \n", y, "\n..ACTUAL: \n", x)
+      stopi("We get a warning as expected but the messages don't match:",
+            actual, "{''c ? m}", expected, y)
     }
   } else if(type == "err"){
     # we expect an error
     m = tryCatch(x, error = function(e) structure(conditionMessage(e), class = "try-error"))
     if(!"try-error" %in% class(m)){
       stop("Expected an error that did not occur.")
+      
     } else if(IS_Y && !grepl(tolower(y), tolower(m), fixed = TRUE)){
-      stop("This is an error, but the messages don't match:\nEXPECTED: \n", y, "\n..ACTUAL: \n", x)
+      stopi("This is an error, but the messages don't match:",
+            actual, "{''c ? m}", expected, y)
     }
+    
   } else if(length(x) == 0){
     stop("Argument 'x' is of length 0. This is not allowed.")
 
+  } else if(IS_Y && inherits(y, "formula")){
+    x_clean = x
+    attributes(x_clean) = NULL
+    
+    y_clean = y
+    attributes(y_clean) = NULL
+    
+    if(!identical(x_clean, y_clean)){
+      stop("The two formulas differ:",
+           actual, deparse_long(x),
+           expected, deparse_long(y))
+    }
+    
   } else if(type %in% c("equal", "=")){
     if(length(x) != length(y)){
-      stop("Lengths differ: ", 
-            "\nEXPECTED: ", length(y), "\n",
-            "\n..ACTUAL: ", length(x))
+      stopi("Lengths differ: {len ? x} vs {len ? y}.", fmt_xy(x, y))
 
     } else if(anyNA(x)){
+      
       if(!all(is.na(x) == is.na(y))){
         if(sum(is.na(x)) != sum(is.na(y))){
-          stop("Number of NA values differ: ",
-          "\nEXPECTED: ", sum(is.na(y)), "\n",
-          "\n..ACTUAL: ", sum(is.na(x)))
+          stopi("Number of NA values differ: {n ? sum(is.na(x))} vs {n ? sum(is.na(y))}.",
+                fmt_xy(x, y))
         }
 
-        i = which(is.na(x) != is.na(y))[1]
+        pos_na = which(is.na(x) != is.na(y))
 
-        na_y = 1 + is.na(y)[i]
-        info = c("none", "one NA")
-
-        stop("Position of the NA values differ. ", 
-              "\nEXPECTED: ", info[na_y], " in position ", i, "\n",
-              "\n..ACTUAL: ", info[3 - na_y], ".")
+        stopi("NA values differ in position{$s} {n, enum.3 ? pos_na}.", 
+              fmt_xy(x, y))
 
       } else if(!all(is_na <- is.na(x)) && any(qui_pblm <- x[!is_na] != y[!is_na])){
 
         if(all(qui_pblm)){
           if(length(x) == 1){
-              stop("Non-NA values differ: ", 
-                    "\nEXPECTED: ", y[!is_na], "\n",
-                    "\n..ACTUAL: ", x[!is_na])
+            stopi("The two values differ: {x} vs {y}")
           } else {
-              stop("All non-NA values differ: 1st elem.: ", 
-                    "\nEXPECTED: ", y[!is_na][1], "\n",
-                    "\n..ACTUAL: ", x[!is_na][1])
+            stopi("All values differ:", fmt_xy(x, y))
           }
         } else {
-          n = sum(qui_pblm)
-          i = which(qui_pblm)[1]
-          stop(n, " non-NA value", plural(n, "s.differ"), ": ", n_th(i), " elem.: \nEXPECTED: ", y[!is_na][i], "\n..ACTUAL: ", x[!is_na][i])
+          n_pblm = sum(qui_pblm)
+          stopi("{n ? n_pblm} non-NA value{#s} differ in position{#s} {which, enum ? qui_pblm}.", 
+                fmt_xy(x, y))
         }
       }
 
     } else if(anyNA(y)){
-      stop("Number of NA values differ: ", 
-            "\nEXPECTED: ", sum(is.na(y)), "\n",
-            "\n..ACTUAL: ", sum(is.na(x)))
+      stopi("Number of NA values differ: {n ? sum(is.na(x))} vs {n ? sum(is.na(y))}.",
+            fmt_xy(x, y))
 
     } else if(!all(x == y)){
       qui_pblm = x != y
 
       if(all(qui_pblm)){
         if(length(x) == 1){
-          stop("Values differ: ", 
-                "\nEXPECTED: ", y, "\n",
-                "\n..ACTUAL: ", x)
+          stopi("The two values differ: {x} vs {y}")
         } else {
-          stop("All values differ: 1st elem.: ", 
-                "\nEXPECTED: ", y[1], "\n",
-                "\n..ACTUAL: ", x[1])
+          stopi("All values differ:", fmt_xy(x, y))
         }
       } else {
-        n = sum(qui_pblm)
-        i = which(qui_pblm)[1]
-        stop(n, " value", plural(n, "s.differ"), ": ", n_th(i), " elem.: \nEXPECTED: ", y[i], "\n..ACTUAL: ", x[i])
+        n_pblm = sum(qui_pblm)
+        stopi("{n ? n_pblm} non-NA value{#s} differ in position{#s} {which, enum ? qui_pblm}.", 
+              fmt_xy(x, y))
       }
     }
+    
   } else if(type %in% c("~", "approx")){
     if(length(x) != length(y)){
-      stop("Lengths differ: ", 
-            "\nEXPECTED: ", length(y), "\n",
-            "\n..ACTUAL: ", length(x))
+      stopi("Lengths differ: {len ? x} vs {len ? y}.", fmt_xy(x, y))
 
     } else if(anyNA(x)){
       if(!all(is.na(x) == is.na(y))){
         if(sum(is.na(x)) != sum(is.na(y))){
-          stop("Number of NA values differ: ", 
-                "\nEXPECTED: ", sum(is.na(y)), "\n",
-                "\n..ACTUAL: ", sum(is.na(x)))
+          stopi("Number of NA values differ: {n ? sum(is.na(x))} vs {n ? sum(is.na(y))}.",
+                fmt_xy(x, y))
         }
 
-        i = which(is.na(x) != is.na(y))[1]
+        pos_na = which(is.na(x) != is.na(y))
 
-        na_y = 1 + is.na(y)[i]
-        info = c("none", "one NA")
-
-        stop("Position of the NA values differ. ", 
-              "\nEXPECTED: ", info[na_y], " in position ", i, "\n",
-              "\n..ACTUAL: ", info[3 - na_y], ".")
+        stopi("NA values differ in position{$s} {n, enum.3 ? pos_na}.", 
+              fmt_xy(x, y))
 
       } else if(max(abs(x - y), na.rm = TRUE) > tol){
-        stop("Difference > tol: Max abs. diff: ", max(abs(x - y), na.rm = TRUE), " (in position ", which.max(abs(x - y)), ")")
+        qui_pblm = abs(x - y) > tol & !is.na(x)
+        stopi("{n ? sum(qui_pblm)} value{#s, differ} in position {which, enum.3 ? qui_pblm}.",
+              fmt_xy(x, y),
+              "\nMax abs. diff: ", max(abs(x - y), na.rm = TRUE))
       }
+      
     } else if(anyNA(y)){
-      stop("Number of NA values differ: ", 
-            "\nEXPECTED: ", sum(is.na(y)), "\n",
-            "\n..ACTUAL: ", sum(is.na(x)))
+      stopi("Number of NA values differ: {n ? sum(is.na(x))} vs {n ? sum(is.na(y))}.",
+            fmt_xy(x, y))
 
     } else if(max(abs(x - y)) > tol){
-      stop("Difference > tol: Max abs. diff: ", max(abs(x - y)), " (in position ", which.max(abs(x - y)), ")")
+      
+      qui_pblm = abs(x - y) > tol & !is.na(x)
+      
+      if(all(qui_pblm)){
+        extra = ""
+        diff = max(abs(x - y))
+        if(diff < 1){
+          extra = sma("\nMax absolute difference: {diff}")
+        }
+        
+        if(length(x) == 1){
+          stopi("The two values differ: {x} vs {y}.", extra)
+        } else {
+          stopi("All values differ.", fmt_xy(x, y), extra)
+        }
+        
+      }
+      
+      
+      stopi("{n ? sum(qui_pblm)} value{#s, differ} in position {which, enum.3 ? qui_pblm}.",
+            fmt_xy(x, y),
+            "\nMax abs. diff: ", max(abs(x - y)))
     }
   }
 
 }
 
 
-chunk = function(x) message(toupper(x), "\n")
+chunk = function(x){
+  options(TEST_CHUNK = x)
+  message(toupper(x), "\n")
+}
 
 test_contains = function(x, pattern){
   update_test_counter()
@@ -278,6 +295,7 @@ test_err_contains = function(x, pattern){
 
 run_tests = function(chunk, from = 1, source = FALSE, use_devtools = TRUE){
   update_test_counter(reset = TRUE)
+  options(TEST_CHUNK = NULL)
   
   wd = getwd()
   
@@ -487,6 +505,11 @@ run_tests = function(chunk, from = 1, source = FALSE, use_devtools = TRUE){
 
     message(file_info, "Test failed at line: ", line_in_file)
     
+    chunk_info = getOption("TEST_CHUNK")
+    if(!is.null(chunk_info)){
+      message("Chunk = ", chunk_info, ", rerun with run_tests(\"", chunk_info, "\")")
+    }
+    
     i = which(test_code == paste0("LINE_COUNTER = ", line_fail))
     my_line = test_code[i + 1]
     message("\nHere is the line:\n", line_in_file, "| ", my_line, "\n")
