@@ -29,17 +29,8 @@
  *                                                                   *
  ********************************************************************/
 
-#include <Rcpp.h>
+#include "fixest_main.h"
 #include <math.h>
-#include <vector>
-#include <cstdint>
-#ifdef _OPENMP
-  #include <omp.h>
-#else
-  #define omp_get_thread_num() 0
-#endif
-
-// [[Rcpp::plugins(openmp)]]
 
 using namespace Rcpp;
 using std::vector;
@@ -70,23 +61,6 @@ using std::int64_t;
  */
 
 
-
-
-
-// Stopping / continuing criteria:
-// Functions used inside all loops
-inline bool continue_crit(double a, double b, double diffMax){
-  // continuing criterion of the algorithm
-  double diff = fabs(a - b);
-  return ( (diff > diffMax) && (diff/(0.1 + fabs(a)) > diffMax) );
-}
-
-inline bool stopping_crit(double a, double b, double diffMax){
-  // stopping criterion of the algorithm
-  double diff = fabs(a - b);
-  return ( (diff < diffMax) || (diff/(0.1 + fabs(a)) < diffMax) );
-}
-
 //
 // Demeans each variable in input
 // The method is based on obtaining the optimal cluster coefficients
@@ -103,165 +77,6 @@ int pending_interrupt() {
 // BUT I need to add static allocation of threads => performance cost
 
 
-//
-// We introduce a class that handles varying types of SEXP and behaves as a regular matrix
-//
-
-
-// Later I may use the number of observations
-// I don't at the moment because everything is strongly checked beforehand
-class sVec{
-  double *p_dble = nullptr;
-  int *p_int = nullptr;
-
-public:
-  // several constructors
-
-  // is_int public member
-  bool is_int = false;
-
-  sVec(){};
-  sVec(SEXP);
-  sVec(double *p_x): p_dble(p_x), is_int(false){};
-  sVec(int *p_x): p_int(p_x), is_int(true){};
-  sVec(std::nullptr_t){};
-
-
-  double operator[](int i){
-    if(is_int) return static_cast<double>(p_int[i]);
-    return p_dble[i];
-  }
-
-};
-
-sVec::sVec(SEXP x){
-  if(TYPEOF(x) == REALSXP){
-    is_int = false;
-    p_dble = REAL(x);
-  } else if(TYPEOF(x) == INTSXP){
-    is_int = true;
-    p_int = INTEGER(x);
-  } else {
-    stop("The current SEXP type is not supported by the sVec class.");
-  }
-}
-
-
-class sMat{
-
-  std::vector<sVec> p_sVec;
-  int n = 0;
-  int K = 0;
-
-  sMat() = delete;
-
-public:
-  sMat(SEXP, bool);
-
-  int nrow(){return n;};
-  int ncol(){return K;};
-
-  sVec operator[](int);
-  double operator()(int, int);
-};
-
-sMat::sMat(SEXP x, bool single_obs = false){
-
-  if(TYPEOF(x) == VECSXP){
-    // x can be a list of either vectors or matrices
-
-    int L = Rf_length(x);
-
-    for(int l=0 ; l<L ; ++l){
-      SEXP xx = VECTOR_ELT(x, l);
-      SEXP dim = Rf_getAttrib(xx, R_DimSymbol);
-
-      int n_tmp = 0, K_tmp = 0;
-
-      if(Rf_length(dim) == 0){
-        // vector
-        n_tmp = Rf_length(xx);
-        K_tmp = 1;
-      } else {
-        int *pdim = INTEGER(dim);
-        n_tmp = pdim[0];
-        K_tmp = pdim[1];
-      }
-
-      // we set the number of rows at the first iteration
-      if(l == 0){
-        n = n_tmp;
-      } else {
-        if(n != n_tmp) stop("When setting up the class sMat: The number of observations in the list is not coherent across columns.");
-      }
-
-      K += K_tmp;
-
-      if(TYPEOF(xx) == REALSXP){
-        double *p_x = REAL(xx);
-        for(int k=0 ; k<K_tmp ; ++k){
-          p_sVec.push_back(sVec(p_x));
-          if(k + 1 < K_tmp) p_x += n;
-        }
-
-      } else if(TYPEOF(xx) == INTSXP){
-        int *p_x = INTEGER(xx);
-        for(int k=0 ; k<K_tmp ; ++k){
-          p_sVec.push_back(sVec(p_x));
-          if(k + 1 < K_tmp) p_x += n;
-        }
-      } else {
-        stop("The current SEXP type is not supported by the sMat class.");
-      }
-    }
-
-
-  } else {
-    // Matrix or vector
-
-    SEXP dim = Rf_getAttrib(x, R_DimSymbol);
-
-    if(Rf_length(dim) == 0){
-      // vector
-      n = Rf_length(x);
-      K = 1;
-    } else {
-      const int *pdim = INTEGER(dim);
-      n = pdim[0];
-      K = pdim[1];
-    }
-
-    if(!single_obs && (n == 1 && K == 1)){
-      // => absence of data
-      n = 0;
-      K = 0;
-
-    } else if(TYPEOF(x) == REALSXP){
-      double *p_x = REAL(x);
-      for(int k=0 ; k<K ; ++k){
-        p_sVec.push_back(sVec(p_x));
-        if(k + 1 < K) p_x += n;
-      }
-
-    } else if(TYPEOF(x) == INTSXP){
-      int *p_x = INTEGER(x);
-      for(int k=0 ; k<K ; ++k){
-        p_sVec.push_back(sVec(p_x));
-        if(k + 1 < K) p_x += n;
-      }
-    } else {
-      stop("The current SEXP type is not supported by the sMat class.");
-    }
-  }
-}
-
-sVec sMat::operator[](int k){
-  return p_sVec[k];
-}
-
-double sMat::operator()(int i, int k){
-  return p_sVec[k][i];
-}
 
 
 
@@ -333,7 +148,7 @@ class FEClass{
   // eq_systems_VS_C: vector stacking all the systems of equations (each system is of size n_coef * n_vs * n_vs)
   // p_eq_systems_VS_C: pointer to the right equation system. Of length Q.
   vector<int*> p_fe_id;
-  vector<sVec> p_vs_vars;
+  vector<RealVec> p_vs_vars;
   double *p_weights = nullptr;
 
   vector<bool> is_slope_Q;
@@ -347,7 +162,7 @@ class FEClass{
   vector<int> coef_start_Q;
 
   // internal functions
-  void compute_fe_coef_internal(int, double *, bool, sVec, double *, double *);
+  void compute_fe_coef_internal(int, double *, bool, RealVec, double *, double *);
   void compute_fe_coef_2_internal(double *, double *, double *, bool);
   void add_wfe_coef_to_mu_internal(int, double *, double *, bool);
 
@@ -356,7 +171,7 @@ public:
   // Utility class: Facilitates the access to the VS variables
   class simple_mat_of_vs_vars{
     int K_fe;
-    vector<sVec> pvars;
+    vector<RealVec> pvars;
 
   public:
     simple_mat_of_vs_vars(const FEClass*, int);
@@ -370,7 +185,7 @@ public:
   FEClass(int n_obs, int Q, SEXP r_weights, SEXP fe_id_list, SEXP r_nb_id_Q, SEXP table_id_I, SEXP slope_flag_Q, SEXP slope_vars_list);
 
   // functions
-  void compute_fe_coef(double *fe_coef, sVec &mu_in_N);
+  void compute_fe_coef(double *fe_coef, RealVec &mu_in_N);
   void compute_fe_coef(int q, double *fe_coef, double *sum_other_coef_N, double *in_out_C);
 
   void add_wfe_coef_to_mu(int q, double *fe_coef_C, double *out_N);
@@ -380,7 +195,7 @@ public:
 
   void add_2_fe_coef_to_mu(double *fe_coef_a, double *fe_coef_b, double *in_out_C, double *out_N, bool update_beta);
 
-  void compute_in_out(int q, double *in_out_C, sVec &in_N, double *out_N);
+  void compute_in_out(int q, double *in_out_C, RealVec &in_N, double *out_N);
 };
 
 FEClass::FEClass(int n_obs, int Q, SEXP r_weights, SEXP fe_id_list, SEXP r_nb_id_Q, SEXP table_id_I, SEXP slope_flag_Q, SEXP slope_vars_list){
@@ -542,7 +357,7 @@ FEClass::FEClass(int n_obs, int Q, SEXP r_weights, SEXP fe_id_list, SEXP r_nb_id
 
     // A) Meta variables => the ones containing the main information
 
-    sMat m_slopes(slope_vars_list);
+    RealMat m_slopes(slope_vars_list);
     p_vs_vars.resize(nb_slopes);
     for(int v=0 ; v<nb_slopes ; ++v){
       p_vs_vars[v] = m_slopes[v];
@@ -626,7 +441,7 @@ FEClass::FEClass(int n_obs, int Q, SEXP r_weights, SEXP fe_id_list, SEXP r_nb_id
 
 
 // Overloaded versions
-void FEClass::compute_fe_coef(double *fe_coef_C, sVec &mu_in_N){
+void FEClass::compute_fe_coef(double *fe_coef_C, RealVec &mu_in_N){
   // mu: length n_obs, vector giving sum_in_out
   // fe_coef: vector receiving the fixed-effects coefficients
 
@@ -643,7 +458,7 @@ void FEClass::compute_fe_coef(int q, double *fe_coef_C, double *sum_other_coef_N
 
 }
 
-void FEClass::compute_fe_coef_internal(int q, double *fe_coef_C, bool is_single, sVec mu_in_N, 
+void FEClass::compute_fe_coef_internal(int q, double *fe_coef_C, bool is_single, RealVec mu_in_N, 
                                        double *sum_other_coef_N, double *in_out_C){
   // mu: length n_obs, vector giving sum_in_out
   // fe_coef: vector receiving the fixed-effects coefficients
@@ -1014,7 +829,7 @@ void FEClass::add_2_fe_coef_to_mu(double *fe_coef_a, double *fe_coef_b, double *
 }
 
 
-void FEClass::compute_in_out(int q, double *in_out_C, sVec &in_N, double *out_N){
+void FEClass::compute_in_out(int q, double *in_out_C, RealVec &in_N, double *out_N){
   // q: the index of the FE
   // *in_out_C: pointer to the vector of cumulated (input-output), of length C (nber of FE coefs)
   //            => this is a precomputation of the target quantity
@@ -1105,7 +920,7 @@ struct PARAM_DEMEAN{
   int *p_iterations_all;
 
   // vectors of pointers
-  vector<sVec> p_input;
+  vector<RealVec> p_input;
   vector<double*> p_output;
 
   // saving the fixed effects
@@ -1230,7 +1045,7 @@ void demean_single_1(int v, PARAM_DEMEAN* args){
   // loading the data
   int nb_coef_T = args->nb_coef_T;
 
-  vector<sVec> &p_input = args->p_input;
+  vector<RealVec> &p_input = args->p_input;
   vector<double*> &p_output = args->p_output;
 
   // fe_info
@@ -1250,7 +1065,7 @@ void demean_single_1(int v, PARAM_DEMEAN* args){
   }
 
   // the input & output
-  sVec &input = p_input[v];
+  RealVec &input = p_input[v];
   double *output = p_output[v];
 
   // We compute the FEs
@@ -1302,9 +1117,9 @@ bool demean_acc_gnl(int v, int iterMax, PARAM_DEMEAN *args, bool two_fe = false)
   }  
 
   // input output
-  vector<sVec> &p_input = args->p_input;
+  vector<RealVec> &p_input = args->p_input;
   vector<double*> &p_output = args->p_output;
-  sVec &input = p_input[v];
+  RealVec &input = p_input[v];
   double *output = p_output[v];
 
   // temp var:
@@ -1618,14 +1433,14 @@ List cpp_demean(SEXP y, SEXP X_raw, SEXP r_weights, int iterMax, double diffMax,
   int Q = Rf_length(r_nb_id_Q);
 
   // info on y
-  sMat m_y(y);
+  RealMat m_y(y);
   int n_vars_y = m_y.ncol();
   bool useY = n_vars_y > 0;
   bool is_y_list = n_vars_y > 1 || TYPEOF(y) == VECSXP;
   int n_obs = m_y.nrow();
 
   // info on X
-  sMat m_X(X_raw);
+  RealMat m_X(X_raw);
   int n_vars_X = m_X.ncol();
   if(useY == false){
     n_obs = m_X.nrow();
@@ -1636,11 +1451,11 @@ List cpp_demean(SEXP y, SEXP X_raw, SEXP r_weights, int iterMax, double diffMax,
     // The data set is of length 1!!!!
     n_obs = 1;
     
-    m_y = sMat(y, true);
+    m_y = RealMat(y, true);
     n_vars_y = m_y.ncol();
     useY = true;
     
-    m_X = sMat(X_raw, true);
+    m_X = RealMat(X_raw, true);
     n_vars_X = m_X.ncol();
     useX = n_vars_X > 0;
     
@@ -1677,7 +1492,7 @@ List cpp_demean(SEXP y, SEXP X_raw, SEXP r_weights, int iterMax, double diffMax,
     p_output[v] = p_output[v - 1] + n_obs;
   }
 
-  vector<sVec> p_input(n_vars);
+  vector<RealVec> p_input(n_vars);
 
   for(int k=0 ; k<n_vars_X ; ++k){
     p_input[k] = m_X[k];
@@ -1781,7 +1596,7 @@ List cpp_demean(SEXP y, SEXP X_raw, SEXP r_weights, int iterMax, double diffMax,
   int ncol = useX ? n_vars_X : 1;
   NumericMatrix X_demean(nrow, ncol);
 
-  sVec p_input_tmp;
+  RealVec p_input_tmp;
   double *p_output_tmp;
   for(int k=0 ; k<n_vars_X ; ++k){
     p_input_tmp = p_input[k];
@@ -1856,111 +1671,6 @@ List cpp_demean(SEXP y, SEXP X_raw, SEXP r_weights, int iterMax, double diffMax,
   return(res);
 }
 
-//
-// Next version => clean c++ code, use only sMat, create file with common functions
-//
 
 
-std::vector<int> set_parallel_scheme_ter(int N, int nthreads){
-  // => this concerns only the parallel application on a 1-Dimensional matrix
-  // takes in the nber of observations of the vector and the nber of threads
-  // gives back a vector of the length the nber of threads + 1 giving the start/stop of each threads
-
-  std::vector<int> res(nthreads + 1, 0);
-  double N_rest = N;
-
-  for(int i=0 ; i<nthreads ; ++i){
-    res[i + 1] = ceil(N_rest / (nthreads - i));
-    N_rest -= res[i + 1];
-    res[i + 1] += res[i];
-  }
-
-  return res;
-}
-
-
-// [[Rcpp::export]]
-List cpp_which_na_inf(SEXP x, int nthreads){
-  // x: vector, matrix, data.frame // double or integer
-
-  /*
-   This function takes a matrix and looks at whether it contains NA or infinite values
-   return: flag for na/inf + logical vector of obs that are Na/inf
-   std::isnan, std::isinf are OK since cpp11 required
-   do_any_na_inf: if high suspicion of NA present: we go directly constructing the vector is_na_inf
-   in the "best" case (default expected), we need not construct is_na_inf
-   */
-
-  sMat mat(x);
-
-  int nobs = mat.nrow();
-  int K = mat.ncol();
-  bool anyNAInf = false;
-  bool any_na = false;    // return value
-  bool any_inf = false;   // return value
-
-  /*
-   we make parallel the anyNAInf loop
-   why? because we want that when there's no NA (default) it works as fast as possible
-   if there are NAs, single threaded mode is faster, but then we circumvent with the do_any_na_inf flag
-   */
-
-  // no need to care about the race condition
-  // "trick" to make a break in a multi-threaded section
-
-  std::vector<int> bounds = set_parallel_scheme_ter(nobs, nthreads);
-
-  #pragma omp parallel for num_threads(nthreads)
-  for(int t=0 ; t<nthreads ; ++t){
-    for(int k=0 ; k<K ; ++k){
-      for(int i=bounds[t]; i<bounds[t + 1] && !anyNAInf ; ++i){
-
-        if(mat[k].is_int){
-          if(mat(i, k) == -2147483648.0){
-            anyNAInf = true;
-          }
-        } else if(std::isnan(mat(i, k)) || std::isinf(mat(i, k))){
-          anyNAInf = true;
-        }
-      }
-    }
-  }
-
-  // object to return: is_na_inf
-  LogicalVector is_na_inf(anyNAInf ? nobs : 1);
-
-  if(anyNAInf){
-    #pragma omp parallel for num_threads(nthreads)
-    for(int i=0 ; i<nobs ; ++i){
-      double x_tmp = 0;
-      for(int k=0 ; k<K ; ++k){
-        x_tmp = mat(i, k);
-        if(mat[k].is_int){
-          if(mat(i, k) == -2147483648.0){
-            is_na_inf[i] = true;
-            any_na = true;
-            break;
-          }
-        } else if(std::isnan(x_tmp)){
-          is_na_inf[i] = true;
-          any_na = true;
-          break;
-        } else if(std::isinf(x_tmp)){
-          is_na_inf[i] = true;
-          any_inf = true;
-          break;
-        }
-      }
-    }
-  }
-
-  // Return
-  List res;
-  res["any_na"] = any_na;
-  res["any_inf"] = any_inf;
-  res["any_na_inf"] = any_na || any_inf;
-  res["is_na_inf"] = is_na_inf;
-
-  return res;
-}
 
