@@ -15,9 +15,8 @@
  **********************************************************************************/
 
 
-#include <Rcpp.h>
+#include "fixest_main.h"
 #include <math.h>
-#include <vector>
 #include <stdio.h>
 #include <cmath>
 #include <functional>
@@ -25,7 +24,6 @@
 using namespace Rcpp;
 using namespace std;
 
-// [[Rcpp::plugins(cpp11)]]
 
 // [[Rcpp::export]]
 NumericVector cpp_partialDerivative_other(int iterMax, int Q, int N, double epsDeriv, 
@@ -1447,6 +1445,90 @@ StringVector cpp_escape_markup(SEXP Rstr){
 
 
 
+// [[Rcpp::export]]
+List cpp_which_na_inf(SEXP x, int nthreads){
+  // x: vector, matrix, data.frame // double or integer
+
+  /*
+   This function takes a matrix and looks at whether it contains NA or infinite values
+   return: flag for na/inf + logical vector of obs that are Na/inf
+   std::isnan, std::isinf are OK since cpp11 required
+   do_any_na_inf: if high suspicion of NA present: we go directly constructing the vector is_na_inf
+   in the "best" case (default expected), we need not construct is_na_inf
+   */
+
+  RealMat mat(x);
+
+  int nobs = mat.nrow();
+  int K = mat.ncol();
+  bool anyNAInf = false;
+  bool any_na = false;    // return value
+  bool any_inf = false;   // return value
+
+  /*
+   we make parallel the anyNAInf loop
+   why? because we want that when there's no NA (default) it works as fast as possible
+   if there are NAs, single threaded mode is faster, but then we circumvent with the do_any_na_inf flag
+   */
+
+  // no need to care about the race condition
+  // "trick" to make a break in a multi-threaded section
+
+  std::vector<int> bounds = set_parallel_scheme(nobs, nthreads);
+
+  #pragma omp parallel for num_threads(nthreads)
+  for(int t=0 ; t<nthreads ; ++t){
+    for(int k=0 ; k<K ; ++k){
+      for(int i=bounds[t]; i<bounds[t + 1] && !anyNAInf ; ++i){
+
+        if(mat[k].is_int){
+          if(mat(i, k) == -2147483648.0){
+            anyNAInf = true;
+          }
+        } else if(std::isnan(mat(i, k)) || std::isinf(mat(i, k))){
+          anyNAInf = true;
+        }
+      }
+    }
+  }
+
+  // object to return: is_na_inf
+  LogicalVector is_na_inf(anyNAInf ? nobs : 1);
+
+  if(anyNAInf){
+    #pragma omp parallel for num_threads(nthreads)
+    for(int i=0 ; i<nobs ; ++i){
+      double x_tmp = 0;
+      for(int k=0 ; k<K ; ++k){
+        x_tmp = mat(i, k);
+        if(mat[k].is_int){
+          if(mat(i, k) == -2147483648.0){
+            is_na_inf[i] = true;
+            any_na = true;
+            break;
+          }
+        } else if(std::isnan(x_tmp)){
+          is_na_inf[i] = true;
+          any_na = true;
+          break;
+        } else if(std::isinf(x_tmp)){
+          is_na_inf[i] = true;
+          any_inf = true;
+          break;
+        }
+      }
+    }
+  }
+
+  // Return
+  List res;
+  res["any_na"] = any_na;
+  res["any_inf"] = any_inf;
+  res["any_na_inf"] = any_na || any_inf;
+  res["is_na_inf"] = is_na_inf;
+
+  return res;
+}
 
 
 
